
(*
 * BBSCallz    - Generates a report of callers to the BBS yesterday.
 *               for PCBoard 14.x
 *
 * Copyright 1989 Lana E. Fox, Don Cheeks 07-mar-89
 *
 * This program is provided courtesy of:
 *
 *      The Modem Zone
 *      Middletown OH
 *      running PCBoard 14.0/E3
 *      (513) 424-7529 HST & (513) 424-8347 (2400)
 *
 * Revision history:
 *
 *  3-07-89 = initial coding
 *  3-02-90 = recompiled using TP 5.5
 *
 * Disclaimer
 * ----------
 *
 * We cannot be responsible for any damages resulting from the use or mis-
 * use of this program!
 *
 * If you have any questions, bugs, or suggestions, please contact us at
 * The Modem Zone,  (513) 424-7529 HST or 424-8347
 *
 * Enjoy!     Lana Fox, Don Cheeks, and Mike Schwab
 *
 *)

{$r-,s-}             (* enable range checking *)
{$v-}                (* allow variable length string params *)
{$M 12000,40000,40000}  {Stack, minheap, maxheap}

(*uses DOS,BufIO,MiniCrt;*)

{$i \tinc\prokit.inc}
type
   color_string = string[8];

type bitmap = array[1..5] of byte;   {stub}
     varstring = byte;
     single = array[0..3] of byte;

type
   {binary version of user record}
   user_rec = record
      usernum:     integer;
      name:        string[25];
      city:        string[25];
      date:        char6;
      level:       integer;
      calls:       longint;
      uploads:     longint;
      downloads:   longint;
      confs:       integer;
      key:         longint;
   end;

const
   maxbuff = 10000;

   whoami  = 'BBS Callz';
   version = 'v1.1';

   comfile  = 'BBSCallz';
   title_1 = 'Callers to ';
   title_2 = ' Yesterday ';
   title_3 = 'There were ';
   title_4 = ' callers yesterday.';
   pcb_version = '14.x';

const

   ansi_ccolor:  color_string = 'none';

   ansi_colors:  array[0..8] of color_string =
        ('0', '0;31', '1;32', '1;33', '1;34', '1;35', '1;36', '1;37', '0');

   ansi_default = 0;
   ansi_red     = 1;
   ansi_green   = 2;
   ansi_yellow  = 3;
   ansi_blue    = 4;
   ansi_magenta = 5;
   ansi_cyan    = 6;
   ansi_white   = 7;
   ansi_gray    = 8;

var
   conv_date:      array[1..6] of char;
   ufd:            buffered_file;
   ofd:            text;   {an ascii file to output}
   ogfd:           text;   {an ansi file to output}

   user:           user_rec;
   users:          integer;

   bbs_name:       string;    {name of bbs from .cnf}
   user_file:      string;    {name and location of USERS file}
   blt_file:       string;    {full drive, path and filename of b&w bulletin}
   max_level:      integer;   {max security level to show}
   reports:        string;
   cnf_sw1:        string;

   curr_date:      char6;
   yest_date:      char6;
   tmp1_date:      string;
   tmp2_date:      string;
   date_dd:        integer;
   date_mm:        integer;
   date_yy:        integer;
   temp_dd:        integer;
   temp_mm:        integer;
   temp_yy:        integer;
   yest_dd:        string;
   yest_mm:        string;
   yest_yy:        string;
   leap_year:      boolean;
   Error_Code:     Integer;
   Counter:        integer;

   now_color:      string;
   bottom_line:    integer;                               {Check it out}

(* ------------------------------------------------------------ *)
function color(control: integer): string20;
   {form an ansi color command}
var
   newcolor: string20;

begin

   newcolor := #27'[' + ansi_colors[control] + 'm';

   if newcolor = ansi_ccolor then
      color := ''
   else
   begin
      ansi_ccolor := newcolor;
      color := newcolor;
   end;
end;
(* ------------------------------------------------------------ *)
{color selection macros}
function RED:     string20; begin RED     := color(ansi_RED); end;
function GREEN:   string20; begin GREEN   := color(ansi_GREEN); end;
function YELLOW:  string20; begin YELLOW  := color(ansi_YELLOW); end;
function BLUE:    string20; begin BLUE    := color(ansi_BLUE); end;
function MAGENTA: string20; begin MAGENTA := color(ansi_MAGENTA); end;
function CYAN:    string20; begin CYAN    := color(ansi_CYAN); end;
function WHITE:   string20; begin WHITE   := color(ansi_WHITE); end;
function GRAY:    string20; begin GRAY    := color(ansi_GRAY); end;

(* ------------------------------------------------------------ *)
procedure load_color_constants(name: string65);
   {load a new set of color constants}
var
   fd:   text;
   i:    integer;

begin
   assign(fd,name);
{$i-}
  reset( fd );
  Error_Code := IoResult;
{$i+}

  IF Error_Code <> 0 Then
     exit
   else
      readln(fd);

   for i := 1 to 8 do
      readln(fd,ansi_colors[i]);

   close(fd);
end;
(* ----------------------------------------------------------- *)
procedure check_leap;
var
   temp: integer;
begin
   temp := (atoi(system_yy) MOD 4);
   if system_yy = '00' then
      leap_year := true
   else
      if temp = 0 then
         leap_year := true
   else
      leap_year := false;
end;

(* ----------------------------------------------------------- *)
procedure last_year;
begin
   date_yy := atoi (system_yy);
   temp_yy := (date_yy - 1);
   yest_yy := itoa(temp_yy);
   yest_mm := '12';
   yest_dd := '31';
   tmp2_date := (yest_yy + yest_mm + yest_dd)
end;

(* ----------------------------------------------------------- *)
procedure last_month;
begin
   date_mm := atoi (system_mm);
   temp_mm := (date_mm - 1);
   yest_mm := itoa (temp_mm);
   if yest_mm = '01' then yest_dd := '31';
   if (yest_mm = '02') and leap_year then
      yest_dd := '29'
   else
      if (yest_mm = '02') and NOT leap_year then
         yest_dd := '28';
   if yest_mm = '03' then yest_dd := '31';
   if yest_mm = '04' then yest_dd := '30';
   if yest_mm = '05' then yest_dd := '31';
   if yest_mm = '06' then yest_dd := '30';
   if yest_mm = '07' then yest_dd := '31';
   if yest_mm = '08' then yest_dd := '31';
   if yest_mm = '09' then yest_dd := '30';
   if yest_mm = '10' then yest_dd := '31';
   if yest_mm = '11' then yest_dd := '30';
   if yest_mm = '12' then yest_dd := '31';
   tmp2_date := (system_yy + yest_mm + yest_dd);

end;
(* ----------------------------------------------------------- *)
procedure day_before;
begin
   date_dd := atoi (system_dd);
   temp_dd := (date_dd - 1);
   yest_dd := itoa (temp_dd);
   tmp2_date := (system_yy + system_mm + yest_dd)
end;

(* ----------------------------------------------------------- *)
procedure yesterday;
begin
   if system_dd <> '01' then
      day_before
   else
      if (system_dd = '01') and (system_mm <> '01') then
         last_month
      else
         last_year;
end;

(* ----------------------------------------------------------- *)
procedure usage;
begin
   writeln;
   writeln('Command line:  ',comfile,' ',comfile,'.cnf');
   writeln;
   writeln('   Line 1 of ',comfile,'.cnf is your BBS name');
   writeln('   Line 2 of ',comfile,'.cnf is full drive:\path\filename of USERS file');
   writeln('   Line 3 of ',comfile,'.cnf is full drive:\path\filename of BLT# file');
   writeln('   Line 4 of ',comfile,'.cnf is maximum security level to include in report');
   writeln('   Line 5 of ',comfile,'.cnf is YES if your event.sys EVENT is run at');
   writeln('     midnight or very shortly thereafter, or NO if it''s far from midnight.');
   writeln;
   halt;
end;


(* ---------------------------------------------------------------- *)
procedure load_info;
   {load the latest configuration file}
var
   fd: text;
begin
   assign( fd, paramstr(1));
{$i-}
  reset( fd );
  Error_Code := IoResult;
{$i+}

  IF Error_Code <> 0 Then
     usage
   else
     begin
       readln(fd,bbs_name);
       readln(fd,user_file);
       readln(fd,blt_file);
       readln(fd,max_level);
       readln(fd,cnf_sw1);
       close(fd)
     end;
end;

(* ----------------------------------------------------------- *)
procedure init;
begin

   if paramcount <> 1 then
      usage
   else
      load_info;

   if (max_level = 0) then
      max_level := 99;

   bopen(ufd,user_file,100,sizeof(pcb_user_rec));
   if berr then
   begin
      writeln('Can''t open user file');
      usage
   end;

end;

(* ----------------------------------------------------------- *)
procedure get_user;
var
   pcb:    pcb_user_rec;
   cty:    anystring;
   i:      integer;

begin
   inc(users);

   if (users mod 10) = 0 then
      write(^M,users:5,' users');

   bread(ufd,pcb);
   user.name      := pcb.name;
   user.city      := pcb.city;
   user.date      := pcb.date;
   user.level     := pcb.level;
   user.calls     := pcb.total_calls;
   user.uploads   := pcb.uploads;
   user.downloads := pcb.downloads;

   {normalize the city format}
   cty := user.city;
   for i := 1 to length(cty) do
      if (cty[i] = '.') or (cty[i] = ',') then
         cty[i] := ' ';

   i := pos('  ',cty);
   while i > 0 do
   begin
      delete(cty,i,1);
      i := pos('  ',cty)
   end;

   while cty[length(cty)] = ' ' do
      dec(cty[0]);

   user.city := cty;

end;

(* ----------------------------------------------------------- *)
procedure show_caller;

var
   c : integer;
   i : integer;
   rpt_footer : string;

begin
   c := 7;
   tmp1_date := (system_yy + system_mm + system_dd);
   yesterday;

   for i := 1 to 6 do
      conv_date[i] := ' ';

   for i := 1 to 6 do
      conv_date[i] := tmp1_date[i];
   for i := 1 to 6 do
      curr_date[i] := conv_date[i];

   for i := 1 to 6 do
      conv_date[i] := tmp2_date[i];
   for i := 1 to 6 do
      yest_date[i] := conv_date[i];

   get_user;

   while not beof(ufd) do
    begin
      get_user;
      if (cnf_sw1 = 'NO') and (user.date = curr_date) or (user.date = yest_date) and (user.level <= max_level) then
         begin
            Counter := Counter+1;
            if (c >= 7) then
               c := 1
            else
               c := (c + 1);
            now_color := color(c);
            writeln(ofd,'              ',user.name,
                        user.city,'':38-length(user.city));
            writeln(ogfd,now_color,'              ',user.name,
                        user.city,'':38-length(user.city))
         end else
         if (cnf_sw1 = 'YES') and (user.date = yest_date) and (user.level <= max_level) then
            begin
               Counter := Counter+1;
               if (c >= 7) then
                  c := 1
               else
                  c := (c + 1);
               now_color := color(c);
               writeln(ofd,'              ',user.name,
                           user.city,'':38-length(user.city));
               writeln(ogfd,now_color,'              ',user.name,
                           user.city,'':38-length(user.city))
            end
    end;
   rpt_footer := (title_3 + itoa(counter) + title_4);
   writeln;
   writeln (ofd,'           ');

   writeln (ogfd,'           ');
   writeln(ofd,'':(77-length(rpt_footer)) div 2,' ',rpt_footer);
   writeln(ogfd,RED+'':(84-length(rpt_footer)) div 2,' ',rpt_footer);
end;

(* ----------------------------------------------------------- *)
procedure report_header;

var
   rpt_title: string;

begin
   rpt_title := (title_1 + bbs_name + title_2);
   writeln(ogfd);
   writeln(ofd);
   writeln(ogfd);
   writeln(ofd);
   writeln(ofd,'':(77-length(rpt_title)) div 2,' ',rpt_title);
   writeln(ogfd,RED+'':(84-length(rpt_title)) div 2,' ',rpt_title);
   writeln(ofd);
   writeln(ogfd);
   writeln(ofd,'               Generated by ',whoami,' ',version,', on ',system_date,' at ',system_time);
   writeln(ogfd,WHITE+'               Generated by ',whoami,' ',version,', on ',system_date,' at ',system_time);
   writeln(ofd);
   writeln(ogfd);
   writeln(ofd,'              User Name                City                     ');
   writeln(ofd,'              -----------------------  -------------------------');
   writeln(ogfd,MAGENTA+'              User Name                City                     ');
   writeln(ogfd,WHITE+'              -----------------------  -------------------------');
   writeln(ofd);
   writeln(ogfd);

end;

(* ----------------------------------------------------------- *)
(* main program *)

begin
   load_color_constants('PROCOLOR');
   writeln;
   writeln(whoami,' ',version,' for PCB ',pcb_version,'.');
   writeln('Courtesy of:  L.E.Fox, D.J.Cheeks  and  The Modem Zone, (513) 424-7529.');
   writeln;
   users := 0;
   max_level := 0;
   counter := 0;
   init;

   writeln('Generating reports.');

   assign(ofd,blt_file);
   assign(ogfd,blt_file+'g');
   {$i-}
   rewrite(ofd);
   rewrite(ogfd);
   Error_Code := IoResult;
   {$i+}
   if Error_Code <> 0 then
      begin
         writeln('Can''t open report output');
         usage
      end;

   report_header;
   show_caller;

   bclose(ufd);
   write(^M,'':60,^M);
   writeln('Processing completed on ',users,' user records.');

   writeln(ofd);
   writeln(ogfd);
   writeln(ofd,'Courtesy of:  L.E.Fox, D.J.Cheeks and  The Modem Zone, (513) 424-7529.');
   writeln(ogfd,CYAN+'Courtesy of:  L.E.Fox, D.J.Cheeks and  The Modem Zone, (513) 424-7529.');
   writeln(ofd);
   writeln(ogfd);
   close(ofd);
   close(ogfd);
end.
